import { Injectable } from '@angular/core'

import { HttpClient, HttpHeaders } from '@angular/common/http'
import { Observable, BehaviorSubject, from, throwError, of } from 'rxjs'
import { map, catchError } from 'rxjs/operators'

import { User } from '../_models/user'
import { RoleReadNoUsers, ApiService, Token } from '../openapi'
import { BootstrapService } from './bootstrap.service'
import { ConfigService } from './config.service'

// interface AuthResponse {
//     access_token: string,
//     roles: string[]
// }

@Injectable()
export class AuthenticationService {
    user = new BehaviorSubject<User>(undefined)
    user$ = this.user.asObservable()
    roles: RoleReadNoUsers[] = []

    constructor(
        private _http: HttpClient,
        public api: ApiService,
        public bootstrapService: BootstrapService,
        public configService: ConfigService,
    ) {
        // set token if saved in local storage
        this.user.next(<User>JSON.parse(localStorage.getItem('user')))
    }

    isLoggedIn() : Observable<boolean> {
        if (!this.user.value) {
            return from([false])
        }
        let body = JSON.stringify({
            token: this.user.value.token,
        })
        return this._http.post(
            '/auth/isLoggedIn',
            body,
            {
                headers: new HttpHeaders({ 'Content-Type': 'application/json' })
            }
        ).pipe(
            map(resp => true),
            catchError(
                err => {
                    const userName = this.user.value['userName']
                    this.user.next(undefined)
                    this.roles = []
                    localStorage.removeItem('user')
                    return throwError(
                        () => new Error('Session of user "' + userName + '" expired.')
                    )
                }
            )
        )
    }

    login(username: string, password: string): Observable<Token> {
        const headers = new HttpHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
        // var formData: any = new URLSearchParams()
        // formData.set('username', userName)
        // formData.set('password', password)
        return this.api.loginForAccessTokenApiTokenPost({
            username: username,
            password: password
        }).pipe(map(
            token => {
                localStorage.setItem('token', token.access_token)
                // store jwt token in local storage to keep user logged in between page refreshes
                // localStorage.setItem('user',
                //     JSON.stringify({
                //         userName: username,
                //         token: token,
                //         roles: response.roles,
                //     })
                // )

                this.bootstrapService.get().subscribe(
                    bsData => this.configService.setConf(bsData)
                )
                return token
                // this.roles = response.roles

                // Notify
                // this.user.next(new User(userName, token))

                // return true to indicate successful login
                // return true
            // } else {
            //     this.user.next(undefined)
            //     this.roles = []
            //     // return false to indicate failed login
            //     // return false
            // }
            // return response
            }
        ))
    }

    logout(): boolean {
        // XXX: not completly safe: the server might be down:
        //      We should actually *check* that the logout response is OK and display message
        // clear token remove user from local storage to log user out
        // let has_token: boolean = this.user.value && !!this.user.value.token
        const has_token: boolean = !!localStorage.getItem('token')
        // localStorage.removeItem('user')
        // this.user.next(undefined)
        // this.roles = []

        // Tell server that the user has logged out
        if (has_token) {
            this._http.get('/api/logout').subscribe(response => {})
            localStorage.removeItem('token')
        }
        this.bootstrapService.get().subscribe(
            bsData => this.configService.setConf(bsData)
        )
        return has_token
    }

    logoutAdmin(): void {
    }

    isAuthorized(roles: string[]): Observable<boolean> {
        // Return true if at least one role in given list matches one role of the authenticated user
        if (roles.length == 0) return of(true)
        if (roles.every(role => role == undefined)) return of(true)
        // return this.roles.filter(value => -1 !== roles.indexOf(value.name)).length > 0
        return this.configService.conf.pipe(map(
            conf => conf.bsData?.user?.roles?.filter(value => -1 !== roles.indexOf(value.name)).length > 0
        ))
    }
}
